module Stockman
  module Logic
    class Kit
      class Content
        attr_reader :kit_id, :variant_id, :amount_in_kit

        def initialize(kit_id, variant_id, amount_in_kit)
          @kit_id        = kit_id.to_i
          @variant_id    = variant_id.to_i
          @amount_in_kit = amount_in_kit.to_i
        end

        def variant
          @variant ||= Variant.new(variant_id)
        end

        def warehouse_ids
          @warehouse_ids ||= variant.warehouse_ids
        end

        def stock_levels
          @stock_levels ||= variant.stock_levels
        end

        def physical_amount(warehouse_id)
          stock_level_amount(:physical, warehouse_id)
        end

        def available_amount(warehouse_id)
          stock_level_amount(:available, warehouse_id)
        end

        private

        def stock_level_amount(type, warehouse_id)
          stock_level = stock_levels.find_by_warehouse(warehouse_id)

          return 0 if amount_in_kit.zero? || stock_level.nil?

          stock_level.send(:"#{type}_amount") / amount_in_kit
        end
      end
    end
  end
end
