module Stockman
  module Logic
    class Kit
      class ContentsCollection
        include Enumerable

        attr_reader :kit_id

        def initialize(kit_id)
          @kit_id = kit_id.to_i
        end

        def [](index)
          contents[index]
        end

        def each
          contents.each{ |content| yield content }
        end

        def minimum_physical_amount(warehouse_id)
          minimum_amount(:physical, warehouse_id)
        end

        def minimum_available_amount(warehouse_id)
          minimum_amount(:available, warehouse_id)
        end

        def contents
          @contents ||= load_contents
        end

        private

        def load_contents
          Storage.obtain_hash("kits:#{kit_id}:variants").map do |variant_id, amount_in_kit|
            Content.new(kit_id, variant_id, amount_in_kit)
          end
        end

        def minimum_amount(type, warehouse_id)
          contents.map{|content| content.send(:"#{type}_amount", warehouse_id)}.min
        end
      end
    end
  end
end
