module Stockman
  module Logic
    class Kit
      class StockLevelsCollection
        include Enumerable

        attr_reader :kit_id

        def initialize(kit_id)
          @kit_id = kit_id.to_i
        end

        def [](warehouse_id)
          find_by_warehouse(warehouse_id)
        end

        def find_by_warehouse(warehouse_id)
          stock_levels.detect{ |stock_level| stock_level.warehouse_id == warehouse_id }
        end

        def each
          stock_levels.each{ |stock_level| yield stock_level }
        end

        def empty?
          stock_levels.empty?
        end

        def contents
          @contents ||= ContentsCollection.new(kit_id)
        end

        def stock_levels
          @stock_levels ||= load_stock_levels
        end

        def warehouse_ids
          @warehouse_ids ||= contents.map(&:warehouse_ids).flatten.uniq
        end

        private

        def load_stock_levels
          warehouse_ids.inject([]) do |memo, warehouse_id|
            memo << StockLevel.new(contents, warehouse_id)
          end
        end
      end
    end
  end
end
