module Stockman
  module Logic
    class Product
      class SellablesCollection
        include Enumerable

        attr_reader :product_id

        def initialize(product_id)
          @product_id = product_id
        end

        def sellables
          @sellables ||= load_sellables
        end

        def stock_levels
          @stock_levels ||= sellables.map(&:stock_levels).flatten.map(&:stock_levels).flatten
        end

        def each
          sellables.each{ |sellable| yield sellable }
        end

        private

        def load_sellables
          Storage.obtain_set("products:#{product_id}:sellables").map do |sellable_data|
            SellableFactory.build_sellable(sellable_data)
          end
        end
      end
    end
  end
end
